/**************************************************************************
*  This file is part of the TAL project (Tiny Abstraction Layer)
*
*  Copyright (c) 2016 by Michael Fischer (www.emb4fun.de).
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  16.10.2016  mifi  First Version for the SAM D21 Xplained Pro board.
*  17.10.2016  mifi  Supports only SERCOM3 (PA22,PA23).
**************************************************************************/
#if defined(USE_BOARD_SAMD21XPRO)
#define __TALBOARD_C__

/*=======================================================================*/
/*  Include                                                              */
/*=======================================================================*/
#include "tal.h"

/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/

/*************************************************************************/
/*  SERCOMWaitBusy                                                       */
/*                                                                       */
/*  Wait for the synchronization of registers between the                */ 
/*  clock domains is completed.                                          */
/*                                                                       */
/*  In    : pDCB                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void SERCOMWaitBusy (SercomUsart *pUSARTx)
{
   while(pUSARTx->SYNCBUSY.reg != 0)
   {
      __asm__ volatile ("nop");
   }
} /* SERCOMWaitBusy */

/*=======================================================================*/
/*  All code exported                                                    */
/*=======================================================================*/

/*************************************************************************/
/*  tal_BoardEnableCOMx                                                  */
/*                                                                       */
/*  Board and hardware depending functionality to enable the COM port.   */
/*  If this port is not supported by the board, return an error.         */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: TAL_OK / error cause                                         */
/*************************************************************************/
TAL_RESULT tal_BoardEnableCOM1 (void)
{
   return(TAL_ERR_COM_PORT_NOHW);
} /* tal_BoardEnableCOM1 */

TAL_RESULT tal_BoardEnableCOM2 (void)
{
   return(TAL_ERR_COM_PORT_NOHW);
} /* tal_BoardEnableCOM2 */

TAL_RESULT tal_BoardEnableCOM3 (void)
{
   return(TAL_ERR_COM_PORT_NOHW);
} /* tal_BoardEnableCOM3 */

TAL_RESULT tal_BoardEnableCOM4 (void)
{
   SercomUsart *pUSARTx = (SercomUsart*)SERCOM3_BASE;

   /*
    * Use SERCOM3, set PA22 to output (TX) and PA23 to input (RX)
    */
    
   /* Enable PORT clock */
   PM->APBBMASK.reg |= PM_APBBMASK_PORT;
   
   /* Set PA22 to output */
   PORTA->DIRSET.reg = (1<<22);
   
   /* Set PA23 to input */
   PORTA->DIRCLR.reg = (1<<23);
   
   /* Set PA22 and PA23 to alternative function */
   PORTA->PINCFG[22].reg = 0x01;  /* Enable MUXEN */
   PORTA->PINCFG[23].reg = 0x01;  /* Enable MUXEN */
   PORTA->PMUX[11].reg   = 0x22;  /* Peripheral function C selected */
   
   /* Use GCLKGEN0 for SERCOM3 */
   GCLK->CLKCTRL.reg = GCLK_CLKCTRL_CLKEN     |
                       GCLK_CLKCTRL_GEN_GCLK0 |
                       GCLK_CLKCTRL_ID_SERCOM3_CORE;  /*lint !e845*/
   
   /* Enable SERCOM3 clock */
   PM->APBCMASK.reg |= PM_APBCMASK_SERCOM3;
   
   /* Reset USART */
   pUSARTx->CTRLA.reg |= SERCOM_USART_CTRLA_SWRST;
   SERCOMWaitBusy(pUSARTx);
   
   /* Use PAD[1] for RX and PAD[0] for TX */
   pUSARTx->CTRLA.reg |= (1<<SERCOM_USART_CTRLA_RXPO_Pos) |
                         (0<<SERCOM_USART_CTRLA_TXPO_Pos);  /*lint !e845*/

   return(TAL_OK);
} /* tal_BoardEnableCOM4 */

TAL_RESULT tal_BoardEnableCOM5 (void)
{
   return(TAL_ERR_COM_PORT_NOHW);
} /* tal_BoardEnableCOM5 */

TAL_RESULT tal_BoardEnableCOM6 (void)
{
   return(TAL_ERR_COM_PORT_NOHW);
} /* tal_BoardEnableCOM6 */

#endif /* USE_BOARD_SAMD21XPRO */

/*** EOF ***/
